// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;
import org.opencv.video.Video;

import edu.wpi.first.cameraserver.CameraServer;
import edu.wpi.first.cscore.CvSink;
import edu.wpi.first.cscore.CvSource;
import edu.wpi.first.cscore.MjpegServer;
import edu.wpi.first.cscore.UsbCamera;
import edu.wpi.first.cscore.VideoMode;
import edu.wpi.first.util.PixelFormat;
import edu.wpi.first.wpilibj.Notifier;
import edu.wpi.first.wpilibj.TimedRobot;
import edu.wpi.first.wpilibj.shuffleboard.BuiltInWidgets;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;

/**
 * The methods in this class are called automatically corresponding to each
 * mode, as described in
 * the TimedRobot documentation. If you change the name of this class or the
 * package after creating
 * this project, you must also update the Main.java file in the project.
 */
public class Robot extends TimedRobot {
    /**
     * This function is run when the robot is first started up and should be used
     * for any
     * initialization code.
     */

    private UsbCamera camera;
    private CvSink sink;
    private CvSource source;
    private Notifier notifier;

    private Mat reusableInput;

    private final double width = 176;
    private final double height = 144;
    private final int fps = 15;

    private final double crosshairLineLength = 50;

    private final Scalar lineColor = new Scalar(0, 255, 0);

    private final Point xStartPoint = new Point(width / 2 - crosshairLineLength / 2, height / 2);
    private final Point xEndPoint = new Point(width / 2 + crosshairLineLength / 2, height / 2);
    private final Point yStartPoint = new Point(width / 2, height / 2 - crosshairLineLength / 2);
    private final Point yEndPoint = new Point(width / 2, height / 2 + crosshairLineLength / 2);

    public Robot() {
        camera = CameraServer.startAutomaticCapture();

        sink = CameraServer.getVideo(camera);

        source = CameraServer.putVideo("Crosshair", 176, 144);

        reusableInput = new Mat();

        //TODO This isn't perfect, because Frame times aren't guaranteed consistent, and processing adds overhead
        // Would be better to do all of this on a coprocessor I'm thinking...
        notifier = new Notifier(() -> {
            sink.grabFrame(reusableInput);

            if (!reusableInput.empty()) {
                Imgproc.line(reusableInput, xStartPoint, xEndPoint, lineColor, 2);
                Imgproc.line(reusableInput, yStartPoint, yEndPoint, lineColor, 2);

                source.putFrame(reusableInput);
            }
        });
        notifier.startPeriodic(1.0/fps);
    }

    @Override
    public void robotPeriodic() {
        
    }

    @Override
    public void autonomousInit() {
    }

    @Override
    public void autonomousPeriodic() {
    }

    @Override
    public void teleopInit() {
    }

    @Override
    public void teleopPeriodic() {
    }

    @Override
    public void disabledInit() {
    }

    @Override
    public void disabledPeriodic() {
    }

    @Override
    public void testInit() {
    }

    @Override
    public void testPeriodic() {
    }

    @Override
    public void simulationInit() {
    }

    @Override
    public void simulationPeriodic() {
    }
}
